"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bodyParser = require("body-parser");
const cors = require("cors");
const express = require("express");
const ExpressRegexTypes_1 = require("./core/utility/ExpressRegexTypes");
const FileSystemFacade_1 = require("./core/utility/FileSystemFacade");
class App {
    constructor(config, app, options) {
        this._expressOptions = options;
        this._config = config;
        this._SetConfig(app);
        this._RequireControllers(app);
    }
    _SetConfig(app) {
        if (!!this._expressOptions.routeConfig)
            this._expressOptions.routeConfig(app);
        app.set("baseRoute", this._config.baseRoute);
        if (this._config.bodyParserEnabled) {
            app.use(bodyParser.json());
        }
        if (this._config.corsEnabled) {
            app.use(cors({ exposedHeaders: this._config.exposedHeaders }));
        }
        if (this._config.serverRenderingEnabled) {
            app.use(express.static('./views'));
            app.set('views', './views');
            app.set('view engine', this._config.serverRenderingEngine);
        }
    }
    _RequireControllers(app) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const dirData = yield FileSystemFacade_1.FileSystemFacade.GetDirData(this._expressOptions.controllerDir);
                dirData
                    .filter((value) => {
                    return value.files.indexOf(this._expressOptions.extension) !== -1;
                })
                    .forEach((value) => {
                    const filesCleaned = value.files
                        .replace(ExpressRegexTypes_1.ExpressRegexTypes.CONTROLLER_EXT, '')
                        .replace('Controller', '');
                    if (ExpressRegexTypes_1.ExpressRegexTypes.CONTROLLER.test(value.files) && !ExpressRegexTypes_1.ExpressRegexTypes.MAP.test(value.files)) {
                        const fullPath = `${value.directory}/${value.files}`;
                        const relativePath = fullPath
                            .replace(this._expressOptions.controllerDir, '')
                            .replace(value.files, '');
                        const ControllerConstructor = require(`${this._expressOptions.controllerDir}/${relativePath}/${value.files}`);
                        const controllerName = Object.keys(ControllerConstructor)[0];
                        const Controller = new ControllerConstructor[controllerName];
                        for (const { method, mountpath, path, requiredBodyParams, fnName } of Controller.$routes) {
                            app.use(mountpath, this._RouterBuild(method, path, fnName, Controller));
                        }
                    }
                });
            }
            catch (e) {
                console.log(e);
            }
        });
    }
    _RouterBuild(method, path, fnName, Controller) {
        const router = express.Router({ mergeParams: this._expressOptions.mergeParams });
        router[method](path, Controller[fnName]);
        return router;
    }
}
App.expressApp = express();
exports.App = App;
//# sourceMappingURL=App.js.map