

//declare type ConfigureCallback = (app: express.Express) => void;

import * as bodyParser from 'body-parser';
import * as cors from 'cors';
import * as express from 'express';

import { Config} from "./core/Config";
import { IConfig }  from "./core/interfaces/IConfig";
import { IExpressOptions } from './core/interfaces/IExpressOptions';
import { ExpressRegexTypes } from './core/utility/ExpressRegexTypes';
import { FileSystemFacade​​ } from './core/utility/FileSystemFacade';
import { IPromiseFileSystem } from './core/interfaces/IPromiseFileSystem';
import { Controller } from './core/Controller';
import { Router } from 'express';

export class App {

  static expressApp: express.Express = express();
  private _config: IConfig;
  private _expressOptions : IExpressOptions;
  private _app : express.Application;

  constructor(config: Config, app: express.Application, options:IExpressOptions) {
    this._expressOptions = options;
    this._config = config;
    this._SetConfig(app);
    this._RequireControllers(app);
  }

  private _SetConfig(app: express.Application): void {
    if(!!this._expressOptions.routeConfig)
        this._expressOptions.routeConfig(app);

        app.set("baseRoute", this._config.baseRoute);
    
        if(this._config.bodyParserEnabled) {
          app.use(bodyParser.json());
        }

        if(this._config.corsEnabled) {
            app.use(cors({ exposedHeaders: this._config.exposedHeaders }));
        }
    
        if(this._config.serverRenderingEnabled){
          app.use(express.static('./views'))
          app.set('views', './views')
          app.set('view engine', this._config.serverRenderingEngine)
        }
  }

  private async _RequireControllers(app: express.Application) {
      
      try {
        const dirData : IPromiseFileSystem[] = await FileSystemFacade​​.GetDirData(this._expressOptions.controllerDir);

        dirData
          .filter((value: IPromiseFileSystem) => {
            return value.files.indexOf(this._expressOptions.extension) !== -1;
          })
          .forEach((value: IPromiseFileSystem) => {
              const filesCleaned: string = value.files
                .replace(ExpressRegexTypes.CONTROLLER_EXT, '')
                .replace('Controller', '');

              if(ExpressRegexTypes.CONTROLLER.test(value.files) && !ExpressRegexTypes.MAP.test(value.files)) {
                  const fullPath = `${value.directory}/${value.files}`;

                  const relativePath = fullPath
                          .replace(this._expressOptions.controllerDir, '')
                          .replace(value.files, '');

                  const	ControllerConstructor: any = require(`${this._expressOptions.controllerDir}/${relativePath}/${value.files}`);
                  const controllerName: string = Object.keys(ControllerConstructor)[0];
                  const Controller: Controller|any​​ = new ControllerConstructor[controllerName];

                  for (const { method, mountpath, path, requiredBodyParams, fnName } of Controller.$routes)
                  {

                    app.use(
                      mountpath,
                      this._RouterBuild(method, path, fnName, Controller)
                    );

                  }

              }
          })
      } catch (e) {
          console.log(e)
      }
  }

  private _RouterBuild(method, path, fnName, Controller) : express.Router {
      const router: express.Router =  express.Router({mergeParams: this._expressOptions.mergeParams});
      
      router[method](path, Controller[fnName]);

      return router;
  }


}
